#!/usr/bin/python
"""scopes.analyze(x)
where x is module, instance, or class
or execute the command line
   python scopes.py module [name1 name2...]"""
import types,os,string
def analyze(c,cs=None,ms=None,d=0):
	#c - a class, instance, module, function, or data
	#cs - table of classes encountered so far
	#ms - table of methods encountered so far
	#d - how many recursive calls, indentation
	if cs==None: cs={}
	if ms==None: ms={}
	if type(c)==types.InstanceType:
		analyzeInstance(c,cs,ms,d)
		return
	if type(c)==types.ModuleType:
		analyzeModule(c,cs,ms,d)
		return
	if type(c)==types.ClassType:
		analyzeClass(c,cs,ms,d)
		return
	if type(c)==types.FunctionType:
		class noClass:pass
		analyzeFunction(c,noClass,ms,d)
		return
	print c,type(c)
	return

def analyzeInstance(c,cs,ms,d):
	print "instance of %s.%s" % ( \
		c.__class__.__module__, \
		c.__class__.__name__)
	print " attributes:"
	for a in c.__dict__.items():
		if type(a[1])==types.ClassType:
			print "  ",a[0],": class",a[1]
		else:
			print "  ",a[0],":",a[1]
	analyzeClass(c.__class__,cs,ms,d+1)
	return

def analyzeModule(c,cs,ms,d):
	print "Module",c.__name__
	showdoc(c,d)
	print " attributes:"
	for a in c.__dict__.items():
		if a[0].startswith("_") : 
			continue
		if type(a[1])==types.ClassType:
			print "  ",a[0],": class",a[1]
		elif type(a[1])!=types.FunctionType:
			print "  ",a[0],":",a[1]
	print " functions:"
	analyzeFunctions(c,{},d+1)
	return

def analyzeClass(c,cs,ms,d):
	print "%sclass %s.%s" % (
		"  "*d,c.__module__,
		c.__name__),
	if cs.get(c): #already encountered
		print " -- already encountered"
	else:
		print
		showdoc(c,d)
		cs[c]=c
		print "  "*d,"attributes of class:"
		count=0
		for a in c.__dict__.items():
			if a[0].startswith("_") : 
				continue
			if type(a[1])==types.FunctionType:
				continue
			count=count+1
			print "  "*d,a[0],":",a[1]			
		if count==0:
			print "  "*d,"    no class attributes"			
		print "  "*d,"functions:"
		analyzeFunctions(c,ms,d)
		for s in c.__bases__:
			analyzeClass(s,cs,ms,d+1)
	return

def analyzeFunction(m,c,ms,d):
		n=m.__name__
		print "%s    %s(%s)" % (
			"  "*d,n,
			stringargs(m.func_code,m.func_defaults)),
		if ms.get(n):
			print "overridden in %s.%s" % (
				ms[n].__module__,
				ms[n].__name__),
			if not issubclass(ms[n],c):
			  print ", NOT A SUBCLASS",
		else:
			ms[n]=c
		print
		showdoc(m,d)
def analyzeFunctions(c,ms,d):
	count=0
	for m in c.__dict__.values() :
		if type(m)!=types.FunctionType:
			continue
		count=count+1
		analyzeFunction(m,c,ms,d)
	if count==0:
		print "  "*d,"  no functions"

def showdoc(o,d):
	if not o.__doc__: return
	L=string.split(o.__doc__,os.linesep)
	for s in L:
		print "  "*d,"    ",s

def stringargs(code,defaults):
	if not defaults: defaults=()
	s=""
	nargs=code.co_argcount
	ndflts=len(defaults)
	firstdflt=nargs-ndflts
	for i in range(nargs):
		s=s+ code.co_varnames[i]
		if i>=nargs-ndflts:
			s=s+"="+repr(defaults[i-firstdflt])
		if i<code.co_argcount-1:
			s=s+","
	i=nargs
	if code.co_flags&0x4:
		if i>0: s=s+','
		s = s + "*" + code.co_varnames[i]
		i=i+1
	if code.co_flags&0x8:
		if i>0: s=s+','
		s = s + "**" + code.co_varnames[i]
	return s

if __name__=='__main__':
	import sys
	exec 'import '+sys.argv[1] + ' as x'
	analyze(x)
	for n in sys.argv[2:]:
		analyze(getattr(x,n))

