""" DEQueue - double-ended queue """
class DEQueueEmptyError(LookupError):pass
class DEQueue:
	"""DEQueue(seq): creates a doubly-ended queue 
	and initializes its contents to the elements 
	of the sequence seq. Parameter seq defaults 
	to an empty sequence."""
	def __init__(self,stuff=()):
		self.rep=[None]*8
		self.hd=0
		self.size=0
		for x in stuff:
			self.put(x)
	def put(self,x):
		"Put at end of a DEQueue: q.put(x)"
		if self.size==len(self.rep):
			self.rep=self.rep[self.hd:]+  \
				self.rep[:self.hd]+  \
				[None]*len(self.rep)
			self.hd=0
		self.rep[(self.hd+self.size)%len(self.rep)]=x
		self.size=self.size+1
	def push(self,x):
		"Push at front of a DEQueue: q.push(x)"
		if self.size==len(self.rep):
			self.rep=self.rep[self.hd:]+  \
				self.rep[:self.hd]+  \
				[None]*len(self.rep)
			self.hd=0
		self.hd=self.hd-1
		if self.hd<0:
			self.hd=self.hd+len(self.rep)
		self.size=self.size+1
		self.rep[self.hd]=x
	def get(self):
		"Get item from front of a DEQueue: q.get()"
		if self.size==0:
			raise DEQueueEmptyError
		x=self.rep[self.hd]
		self.rep[self.hd]=None
		self.hd=(self.hd+1)%len(self.rep)
		self.size=self.size-1
		return x
	def pop(self):
		"Pop item from front of a DEQueue: =q.get()"
		return self.get()
	def pull(self):
		"Get item from end of a DEQueue: q.pull()"
		if self.size==0:
			raise DEQueueEmptyError
		i=(self.hd+self.size-1)%len(self.rep)
		x=self.rep[i]
		self.rep[i]=None
		self.size=self.size-1
		return x
	def copy(self):
		"Copy of a DEQueue: q.copy()"
		x=DEQueue()
		x.rep=self.rep[:]
		x.hd=self.hd
		x.size=self.size
		return x
	def __getitem__(self,i):
		"Look at i-th item: q[i]"
		if i<0: 
			i = self.size + i
		if i<0 or i>=self.size:
			raise IndexError
		return self.rep[(self.hd+i)%len(self.rep)]
	def __len__(self):
		'Return the length of queue: len(q)'
		return self.size
	def __nonzero__(self):
		'Return true if the queue is not empty: if q:'
		return self.size!=0
	def __repr__(self):
		'Return a string showing the contents of the queue: repr(q)'
		j=self.hd+self.size
		if j<=len(self.rep):
			s=self.rep[self.hd:j]
		else:
			s=self.rep[self.hd:]+  \
				self.rep[:j%len(self.rep)]
		return "DEQueue("+repr(s)+")"
	def __str__(self):
		'Return a string showing the contents of the queue: str(q)'
		return repr(self)

