"""IntSet -- sets represented as bit strings"""
import types
def _listTupleType(x):
	tmp = type(x)
	return	tmp==types.ListType or \
		tmp==types.TupleType

def bitsinword(x):
	#print hex(x)
	x=(x & 0x55555555) + ((x>>1) & 0x55555555)
	#print hex(x)
	x=(x & 0x33333333) + ((x>>2) & 0x33333333)
	#print hex(x)
	x=(x & 0x0f0f0f0f) + ((x>>4) & 0x0f0f0f0f)
	#print hex(x)
	#x=(x & 0x00ff00ff) + ((x>>8) & 0x00ff00ff)
	x=(x + (x>>8)) & 0x00ff00ff
	#print hex(x)
	#x=(x & 0x0000ffff) + ((x>>16) & 0x0000ffff)
	x=(x + (x>>16)) & 0x0000ffff
	#print hex(x)
	return x
def bitsinlong(x):
	if x<0: raise OverflowError()		
	count=0
	while x!=0L:
		count = count + \
			bitsinword(int(x&0xffffffffL))
		x=x>>32
	return count

def fullSet(n):
	'''fullSet(n)
	return a set containing all integers 0<=i<n
	'''
	#print n,(1L<<n)-1
	return Set(rep=(1L<<n)-1)

class IntSetEnumeration:
	"""object to return the elements of a set one at a time"""
	def __init__(self,s):
		self.members=s.members()
	def hasMoreElements(self):
		"""returns true if the enumeration has more 
		elements to deliver. After returning false, 
		the SetEnumeration is no longer active and 
		will deliver no more elements. """
		return len(self.members)>0
	def nextElement(self):
		"""returns the next element in the enumeration. 
		Raises IndexError if it is called when no more 
		elements are present. After raising IndexError, 
		the SetEnumeration is no longer active and will 
		deliver no more elements. """
		x=self.members[-1]
		del self.members[-1]
		return x
	def __getitem__(self,i):
		"""equivalent to e.nextElement(). 
		The parameter is ignored."""
		return self.nextElement()
	__nonzero__=hasMoreElements

import SetExceptions
class Set:
	def __init__(self,init=(),rep=None):
		"""Set(), Set(init)
		creates a set, either empty, or containing 
		the integers in sequence init
		"""
		if rep:
			self.rep=long(rep)
			return
		self.rep=0L
		if len(init)==0:return
		for i in init :
			self.rep=self.rep|(1L<<i)
	def __or__(self, other):
		""" __or__(self, other), self|other
		returns the union of self and other
		"""
		return Set(rep=self.rep|other.rep)
	def __and__(self, other):
		""" __and__(self, other), self&other
		returns the intersection of self and other
		"""
		return Set(rep=self.rep&other.rep)
	def __sub__(self, other):
		""" __sub__(self, other), self-other
		returns the relative complement of
		self and other, self&~other
		"""
		return Set(rep=self.rep&~other.rep)
	def issubsetof(self,other):
		"""issubsetof(self,other), self<=other
		returns true of all bits set in self are
		also set in other
		"""
		return (self.rep & ~other.rep) == 0
	def ispropersubsetof(self,other):
		"""ispropersubsetof(self,other), self<other
		returns true of all bits set in self are
		also set in other, but some bits in other
		are not in self
		"""
		return self.issubsetof(other) and \
			(self.rep != other.rep) 
	def equals(self,other):
		"""equals(self,other)
		returns true if both sets contain the same elements
		"""
		return self.rep==other.rep
	def __cmp__(self,other):
		"""__cmp__(self,other)
		returns
			-1 if self is a proper subset of other
			0 if self == other
			1 otherwise
			Note: not symmetric """
		if self.rep==other.rep:
			return 0
		elif (self.rep & ~other.rep) == 0:
			return -1
		else:
			return 1
	def __invert__(self):
		"""__invert__(self), ~self
		returns the complement of a set""" 
		return Set(rep=~self.rep)
	def contains(self,i):
		"""contains(self,i)
		returns true if i is an element of self
		"""
		return int((self.rep>>i) & 1L)
	__contains__=contains
	def with(self,i):
		"""with(self,i)
		returns self|Set([i])
		"""
		return Set(rep=self.rep|(1L<<i))
	def without(self,i):
		"""without(self,i)
		returns self-Set([i])
		"""
		return Set(rep=self.rep&~(1L<<i))
	def insert(self,i):
		"""insert(self,i)
		changes self to include element i
		no operation if self.contains(i)
		"""
		self.rep=self.rep|(1L<<i)
	def delete(self,i):
		"""delete(self,i)
		changes self to not contain i
		no operation if not self.contains(i)
		"""
		self.rep=self.rep&~(1L<<i)
	def __len__(self):
		"""__len__(self), len(self)
		returns the number of elements in the set
		raises OverflowError if there are infinitely many
		"""
		return bitsinlong(self.rep)
	def members(self):
		"""members(self)
		returns a tuple of the elements in the set
		raises InfiniteSetException if there are infinitely 
		many elements
		"""
		if self.rep<0: 
			raise SetExceptions. \
				InfiniteSetException()		
		x=self.rep
		i=0
		L=[]
		while x:
			if x&1L:
				L.append(i)
			x=x>>1
			i=i+1
		return tuple(L)
	def items(self):
 		"""items(self)
		same as self.members()
		"""
		return self.members()
	def elements(self):
		"""elements(self)
		returns an enumeration object to yield the
		elements of the set in a loop
		"""
		return BitSetEnumeration(self)
	def arb(self):
		"""arb(self)
		returns an arbitrary element of the set
		"""
		if self:
			#z&(-z) will clear all but the rightmost
			# bit of z
			#z-1 will clear the rightmost bit and 
			# set all bits to its right
			#the number of bits to the right 
			# equals the position of the bit
			x=self.rep&(-self.rep)
			return bitsinlong(x-1)
		else:
			raise SetExceptions.EmptySetException()
	def __nonzero__(self):
		"""__nonzero__(self)
		equivalent to len(self)!=0
		"""
		return self.rep!=0
	def __getitem__(self,k):
		"""__getitem__(self,k)
		returns the kth element of the set, starting 
		at k==0 and the lowest number in the set
		"""
		if len(self)<=k: raise IndexError()
		i=0
		x=self.rep
		while x:
			if x&1L:
				if k==0: return i
				else: k=k-1
			i=i+1
			x=x>>1
		else: raise IndexError()
	def __repr__(self):
		"""__repr__(self)
		returns a Set() call that will recreate the set
		"""
		return "Set(rep="+hex(self.rep)+")"
	def __str__(self):
		"""__str__(self)
		equivalent to repr(self)
		"""
		return repr(self)
	def copy(self): 
		"""copy(self)
		returns a new set with the same elements as self
		"""
		return Set(rep=self.rep)
	def new(self): 
		"""new(self)
		returns an empty set
		"""
		return Set()

