"""Priority queue
prioque.prioque(before)
prioque.prioque()
creates a priority queue that allows multiple 
occurrences of an item. The before(x,y) function 
is to return true only if priority x is to precede 
priority y. The module contains these functions:
prioque.smallerfirst computes x<y
prioque.largerfirst computes y<x
 Function before() defaults to smallerfirst."""
# these functions determine whether the first
# or second argument should be preferred.
# one of them is passed to parameter "first"
#when the queue is created
def smallerfirst(x,y): return x<y
def largerfirst(x,y): return y<x

class prioque:
	def __init__(self,before=smallerfirst):
		self.q=[(None,None)]
		self.before=before
	def __siftHole(self,i):
		j=i+i
		n=len(self.q)
		while j<n:
			k=j+1
			if k<n and \
			    self.before(self.q[k][0],self.q[j][0]):
				j=k
			self.q[i]=self.q[j]
			i=j
			j=i+i
		return i
	def __siftRootwards(self,i):
		j=i/2
		while j>0:
		    if self.before(self.q[j][0],self.q[i][0]):
			break
		    self.q[i],self.q[j]=self.q[j],self.q[i]
		    i=j
		    j=j/2
		return i
	def put(self,prio,item): 
		'''q.put(priority,item)
		puts the item with the given priority into 
		the queue. The priorities must be comparable 
		values, e.g. cmp() is defined for the priorities.'''
		n=len(self.q)         
		self.q.append((prio,item))
		self.__siftRootwards(n)
	def getPair(self):
		'''q.getPair() 
		removes and returns (p,x) where x is the first 
		item in the queue and p is its priority. It 
		raises an exception if the queue is empty. '''
		if len(self.q)<2: raise LookupError
		item=self.q[1]
		j=self.__siftHole(1)
		self.q[j]=self.q[-1]
		self.__siftRootwards(j)
		del self.q[-1]
		return item
	def get(self):
		'''q.get() removes and returns the first 
		(highest priority) item in the queue. It 
		raises an exception if the queue is empty.'''
		return self.getPair()[1]
	def headPair(self):
		'''q.headPair() returns (p,x) where x is 
		the first item in the queue and p is its 
		priority. It does not remove x. It raises 
		a LookupError exception if the queue is empty.'''
		if len(self.q)<2: raise LookupError
		return self.q[1]
	def head(self):
		'''q.head() returns the first item in the 
		queue without removing it. It raises a 
		LookupError exception if the queue is empty.'''
		if len(self.q)<2: raise LookupError
		return self.q[1][1]
	def __len__(self):
		'''len(q) or q.__len__() returns the number 
		of items in the queue.'''
		return len(self.q)-1
	def __getitem__(self,i):
		'''q[i] or q.__getitem__(i) returns (p,x) 
		where x is the ith item in the queue and p 
		is its priority. The items are not in an 
		obvious order.'''
		return self.q[i+1]
	def __nonzero__(self):
		'''q.__nonzero__() or if q: returns true if 
		len(q)>0, false otherwise. '''
		return len(self.q)>1
