import toolsofcomputing_org.adt.DEQueue as DEQueue
import TraceException, sys
import operator
def countBACKUPs(tables):
	nb=0
	for n in tables.nonterminals:
		if tables.deflt.has_key(n):
			r=tables.deflt[n]
			nb=max(reduce(operator.add, \
				map(lambda x: x=='BACKUP',r),0),nb)
		if tables.sel.has_key(n):
			for t in tables.sel[n].keys():
				r=tables.sel[n][t]
				nb=max(reduce(operator.add, \
					map(lambda x: x=='BACKUP',r)),nb)
	return nb

class LLkParser:
  def __init__(self,scanner,tables,semantics,error,errorRecovery=None):
	self.scanner=scanner
	self.tables=tables
	self.semantics=semantics
	self.parseErrorOutput=error
	if errorRecovery is None:
		import PanicMode
		self.errorRecovery=PanicMode.PanicMode()
	else:
		self.errorRecovery=errorRecovery
	

	self.inputStk=DEQueue.DEQueue()
	self.predictionStk=DEQueue.DEQueue()
	self.actionQueue=DEQueue.DEQueue()
	self.tokenQueue=DEQueue.DEQueue()
	self.maxQueuedTokens=max(2,countBACKUPs(tables))

	self.currentToken=None
	self.top=None
	
  def revisedCopy(self,scanner,tables,semantics,error,errorRecovery=None):
  	c=LLkParser(scanner,tables,semantics,error,errorRecovery=None)
  	c.inputStk=self.inputStk.copy()
  	c.predictionStk=self.predictionStk.copy()
  	c.actionQueue=self.actionQueue.copy()
  	c.tokenQueue=self.tokenQueue.copy()
  	c.currentToken=self.currentToken
  	c.top=self.top
  	return c
  
  def putToken(self,top,t):
	self.actionQueue.put(top)
	self.tokenQueue.put(t)
	if (len(self.tokenQueue)>self.maxQueuedTokens) :
		self.outputNextToken()

  def delPredictionTop(self):
	self.predictionStk.pop()

  def popTop(self): 
	self.top = self.predictionStk.pop()

  def pullTop(self): 
	self.top = self.actionQueue.pull();

  def putCurrentToken(self,top): 
	#self.putToken(self.top, self.currentToken)
	self.putToken(top, self.currentToken)
	self.currentToken = None

  def getToken(self): 
	if not self.inputStk: 
		self.currentToken=self.scanner.scan()
	else: self.currentToken=self.inputStk.pop()

  def scanAhead(self,length):
  	while len(self.inputStk)<length:
  		t=self.scanner.scan()
  		self.inputStk.put(t)
  		if t.type==tables.eoi:
  			break

  def backupToScan(self):
	self.predictionStk.push(self.top)
	#back up to state just after scanning last token
	while self.actionQueue:
		self.pullTop()
		if self.tables.terminals.contains(self.top):
			self.actionQueue.put(self.top)
			break
		elif self.tables.nonterminals.contains(self.top):
			if self.selHasKeys(self.top,self.currentToken.type):
				r=self.tables.sel[self.top] \
					[self.currentToken.type]
 				for j in xrange(len(r)):
					self.delPredictionTop()
			elif self.tables.deflt.has_key(self.top):
				r = self.tables.deflt[self.top]
				for j in xrange(len(r)):
					self.delPredictionTop()
			else: 
				raise "System Error in nonterminals from AQ"
		self.predictionStk.push(self.top)

  def BACKUP(self): 
	if self.currentToken:
		self.inputStk.push(self.currentToken)
		self.currentToken=None
	self.currentToken = self.tokenQueue.pull()
	if not isinstance(self.currentToken,ErrorToken):
		self.inputStk.push(self.currentToken);
	self.currentToken = None
	s=self.actionQueue.pull()
	while not self.tables.terminals.contains(s):
		s=self.actionQueue.pull();

  def performActions(self): 
	while self.actionQueue:
		x=self.actionQueue.get()
		if self.tables.actions.contains(x):
			self.semantics.outAction(x)
		elif self.tables.terminals.contains(x):
			self.semantics.outToken(self.tokenQueue.get());

  def outputNextToken(self): 
	while self.actionQueue:
		x=self.actionQueue.get()
		if self.tables.actions.contains(x):
			self.semantics.outAction(x)
		elif self.tables.terminals.contains(x):
			self.semantics.outToken(self.tokenQueue.get())
			break
		
  def selHasKeys(self,N,T):
	return self.tables.sel.has_key(N) and \
		self.tables.sel[N].has_key(T)

  def showPredictionStack(self,caption): 
	print caption+":",self.predictionStk	

  def parse(self): 
    self.predictionStk.push(self.tables.eoi);
    self.predictionStk.push(self.tables.start);
    try:
	while 1:
		#print self.currentToken,
		#self.showPredictionStack("::")
		if not self.predictionStk:
			self.performActions()
			return self.semantics.close()
		self.popTop()
		while self.tables.actions.contains(self.top):
			if self.top==self.tables.backup:
				self.BACKUP()
			elif self.top==self.tables.doPendingActions:
				self.performActions()
			else: 
				self.actionQueue.put(self.top)
			if not self.predictionStk:
				self.performActions()
				return self.semantics.close()
			self.popTop()
		if self.currentToken==None:
			self.getToken()
		if self.top==self.currentToken.type:
			self.putCurrentToken(self.top)
			if self.top==self.tables.eoi:
				self.performActions()
				return self.semantics.close()
		elif self.selHasKeys(self.top,self.currentToken.type):
			self.actionQueue.put(self.top)
			r=self.tables.sel[self.top][self.currentToken.type]
			for j in range(len(r)-1,-1,-1):
				self.predictionStk.push(r[j])
		elif self.tables.deflt.has_key(self.top):
			self.actionQueue.put(self.top)
			r=self.tables.deflt[self.top]
			for j in range(len(r)-1,-1,-1):
				self.predictionStk.push(r[j])
		else:	#error in input
  			self.backupToScan()
			#recoverFromError is a template method
			if not self.errorRecovery.recoverFromError(self):
				return None
    except Exception,e:
  	cl,exc,tb=sys.exc_info()
	print "exception in parser:"
	TraceException.toRaise(tb)
	print cl,exc

